#!/usr/bin/python
# -*- coding: utf-8 -*-

from panda3d.core import BitMask32, NodePath, Vec3
from panda3d.bullet import BulletCapsuleShape, BulletRigidBodyNode

__author__ = "Fireclaw the Fox"
__license__ = """
Simplified BSD (BSD 2-Clause) License.
See License.txt or http://opensource.org/licenses/BSD-2-Clause for more info
"""


#
# CAMERA FUNCTIONS
#
class Physics:
    def __init__(self):
        # setup the capsule that surrounds the player
        self.player_capsule = BulletCapsuleShape(self.player_radius, self.player_height)

    def startPhysics(self):
        self.player_capsule_np = NodePath(BulletRigidBodyNode("Capsule"))
        self.player_capsule_np.node().addShape(self.player_capsule)
        self.player_capsule_np.node().setKinematic(True)
        self.player_capsule_np.node().setMass(self.player_mass)
        self.player_capsule_np.setCollideMask(BitMask32.allOn())
        self.player_capsule_np.setPos(0, 0, self.player_height)
        self.physic_world.attachRigidBody(self.player_capsule_np.node())
        self.reparentTo(self.player_capsule_np)
        self.player_capsule_np.reparentTo(render)
        self.mainNode = self.player_capsule_np

        self.should_jump = False
        self.jump_time = 0.0

        #taskMgr.add(self.updatePhysics, "task_physics", priority=-15)

    def updatePhysics(self, task):
        dt = globalClock.getDt()
        self.player_capsule_np.setZ(
            self.player_capsule_np,
            -self.physic_world.getGravity().z * dt)

    def updatePlayerPos(self, speed, heading, dt):
        if heading is not None:
            self.player_capsule_np.setH(camera, heading)
            self.player_capsule_np.setP(0)
            self.player_capsule_np.setR(0)
        self.player_capsule_np.setPos(self.player_capsule_np, (0, -speed, 0))
        if self.should_jump:
            self.processJumping(dt)

    def processJumping(self, dt):
        self.jump_time += dt
        self.player_capsule_np.setZ(
            self.player_capsule_np,
            self.jump_strength * self.jump_time)
        if self.jump_time > 0.5:
            self.should_jump = False
            self.jump_time = 0.0
        #if self.__currentPos.z >= self.jumpMaxHeight:
        #    self.__fall()

    def getFirstCollisionInLine(self, pointA, pointB):
        """A simple raycast check which will return the first collision point as
        seen from point A towards pointB"""
        result = self.physic_world.rayTestClosest(pointA, pointB)
        if result.hasHit():
            return result.getHitPos()
        return None

    def doJump(self, speed):
        """This will let the actor jump forward on the local y-axis
        with the upward speed given in jumpForce and forward by the
        given speed."""
        self.should_jump = True
        #jumpVec = Vec3(0, speed, 1)
        #jumpVec *= self.jump_strength
        #self.player_capsule_np.node().applyCentralImpulse(jumpVec)
